/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *  main.c - c file for main function
 *  DRV824x-Q1EVM_FIRMWARE
 *  1/12/2021
 *  updated 1/17/2022 by Murugavel Raju
 *  updated 09/10/2023 by Murugavel Raju
 ******************************************************************************/
#include <msp430.h>
#include "drv824x/drv824x.h"
#include "Serial_Cmd_Monitor.h"

/*
 * Globals
 */
int timerUpdate = 0;
unsigned int OFCount;
volatile char Adcresults = 0;
volatile unsigned int ipropiRaw[64];
extern volatile uint8_t gInterfaceType;        // container for SPI or HW device interface type
extern volatile uint8_t gBridgeType;
extern volatile char prev_in1_duty;
volatile uint8_t exitflag;                     // for tracking push button manual test mode entry and exit for H-variants

void main(void)
{
    // Initialize MCU peripherals
    initMcu();

init:
    if (drv824xq1_init())
    {
        /*
         * Do not pass GO and proceed directly to fail if drv824xq1_init
         * fails.  This means VM is not present or some hardware fault exists.
         * Enable nFAULT as an output to flash the red LED while in this state
         * to indicate a problem exists.  nFAULT will automatically be turned
         * off and reconfigured as an input when initMcu() executes again.
         */
        P2OUT ^= STATUS;                       // toggle STATUS LED
        __delay_cycles(2000000);               // LED on delay time
        goto init;
    }

    firmwareInit();
    _enable_interrupt();
    __bis_SR_register(GIE);

    // GUI Composer Monitor Initialization
    ClearBufferRelatedParam();





    while (1)
    {

        //        P1OUT |= BIT4;  // for TP3 probing with oscilloscope
        gui_deviceControlUpdateTask();
        drv824x_814x_RegistersRead();          // Set trigger for reading drv84x2 Registers
        gui_regUpdateTask();
        ipropi_calc();

        if (!gBrushedObj.startPassiveDiagnostics)
        {
            if (gBrushedObj.startMotor && !gBrushedObj.active)
            {
                /*
                 * Start up DRV
                 */
                drv_startPWM();
            }
            else if (!gBrushedObj.startMotor && gBrushedObj.active)
            {
                /*
                 * Stop motor is pressed
                 */
                if (!TA0CCR1 && !TA1CCR1)
                gBrushedObj.active = 0;
            }

            if (timerUpdate == 1)
            {
                updatePWM();
                timerUpdate = 0;
            }
        }
        else
        {
            TA0CTL |= TASSEL_2 | MC_0 | TACLR; // Stop Timer used for EN_IN1
            TA1CTL |= TASSEL_2 | MC_0 | TACLR; // Stop Timer used for PH_IN2
            PassiveDiagnostics();
        }

        __bis_SR_register(LPM0_bits + GIE);    // Enter LPM0
    }
}

/*
 * ISR routines
 */

/*
 * Timer1_A0 ISR: PHIN2 PWM ISR
 */
#pragma vector=TIMER1_A1_VECTOR
__interrupt void Timer1_A1(void)
{
    timerUpdate = 1;
    /*
     * Checking TA1IV clears the interrupt flag
     */
    switch (__even_in_range(TA1IV, 10))        // Efficient switch-implementation
    {
      case 0:
          break;                               // No interrupt
      case 2:
          break;                               // CCR1 capture interrupt
      case 4:
          break;                               // CCR2 capture interrupt
      case 6:
          break;                               // Reserved
      case 8:
          break;                               // Reserved
      case 10:
          //  P2OUT |= STATUS;
          break;                               // Timer Overflow
    }
}

/*
 * Timer0_A1: ENIN1 PWM ISR
 */
#pragma vector=TIMER0_A1_VECTOR
__interrupt void Timer0_A1(void)
{
   timerUpdate = 1;
   /*
    * Checking TA0IV clears the interrupt flag
    */
    switch (__even_in_range(TA0IV, 10))
    {
      case 0:
          break;                               // No interrupt
      case 2:
          break;                               // CCR1 capture interrupt
      case 4:
          break;                               // CCR2 capture interrupt
      case 6:
          break;                               // Reserved
      case 8:
          break;                               // Reserved
      case 10:
          //P2OUT ^= STATUS;
          break;                               // Timer Overflow
    }
}

/*
 * USCI Receive interrupt to receive the MDBU data packet
 */
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void)
{
    cmdRxIsr();
//  P2OUT &= ~STATUS;
}

/*
 * USCI transmit interrupt to transmit the MDBU data packet
 */
#pragma vector=USCIAB0TX_VECTOR
__interrupt void USCI0TX_ISR(void)
{
//  P2OUT &= ~STATUS;                          // for code debugging
}

/*
 * Status LED derived from the watchdog timer
 */
#pragma vector=WDT_VECTOR
__interrupt void WatchDog_Timer(void)
{
    ++OFCount;

    gBrushedObj.nFault = read_FAULT_pin >> 3;  // update GUI widget indicator

    if (gBrushedObj.nSleep == 1)
    {
        if (gBrushedObj.active && !gBrushedObj.drvoff)
            P2OUT |= STATUS;                   // solid indicates DRV output(s) are active
        else
        {
            if (!(OFCount % 200))
                P2OUT ^= STATUS;               // blinking status indicates DRV is ready
        }
    }
    else
    {
        if (!(OFCount % 800))
            P2OUT ^= STATUS;                   // Blink STATUS slower to indicate DRV is in sleep mode
    }
    __bic_SR_register_on_exit(LPM0_bits);      // Clear LPM0 bits
}

/* ISR for port 2 Interrupts
 * check  for FAULT
 * */
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR(void)
{
    if (P2IFG & nFAULT)                        // BIT3
    {
        gBrushedObj.nFault = read_FAULT_pin >> 3; // nFAULT is asserted --> fault present
        P2IE &= ~nFAULT;                       // disable nFAULT interrupt

//    Code for disabling the driver whenever nFAULT pin goes low - additional safety feature if needed by the application
//    gBrushedObj.drvoff = 1;                    // assert DRVOFF signal
//    gBrushedObj.startMotor = 0;                // signals main loop to stop PWM
//    gBrushedObj.en_in1 = 0;                    // reset PWM duty cycle settings
//    gBrushedObj.ph_in2 = 0;
    }
}

volatile uint8_t prevBridgeMode;

#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR(void)
{
    if (P1IFG & PUSH)                          // BIT3
    {
        P1IFG &= ~PUSH;
        /*
         * Drive OUT1 at 25% duty cycle if not already active
         * Note: this is only intended for testing purposes
         *       and for the sake of simplicity, we're not saving
         *       all state and control variables.  This can cause
         *       issues with getting out of sync with the GUI.
         */
        if (gBrushedObj.startMotor == 0)
        {
            /*
             * Note: MODE_PHEN is used here because it is the
             *       default out-of-box setting on DRV824xH
             *       EVMs.  May need to handle DRV814x devices
             *       different.
             */
            if (gBridgeType == HALF_BRIDGE)    // DRV814x
            {
                gBrushedObj.bridgeMode = MODE_INDEPENDENT;
                gBrushedObj.Loadhighlow = 0;   // Low side load
            }
            else
            {
                prevBridgeMode = gBrushedObj.bridgeMode;
                gBrushedObj.bridgeMode = MODE_PHEN;
            }

            if (gInterfaceType != SPI_INTERFACE)
            {                                  // HW variant
                if (read_nSLEEP_pin == 0)
                {
                    exitflag = 1;
                    set_nSLEEP_hi;             // invoke wake up from sleep
                    gBrushedObj.nSleep = 1;    // for GUI
                    __delay_cycles(8000);      // > t0+t1, 400+22 = 422us, value set for 500us
                    while (read_FAULT_pin == 1); // verify and wait until nFAULT = 0
                    set_nSLEEP_lo;             // nSLEEP reset pulse issued
                    __delay_cycles(RESET_TIMEOUT);// duration for nSLEEP reset pulse for HW variants
                    set_nSLEEP_hi;             // nSLEEP reset pulse completed
                    //
                    while (read_FAULT_pin == 0);// wait until nFAULT = 1
                }
            }

            gBrushedObj.drvoff = 0;
            // for direction change ph = 1 forward, ph = 0 reverse
            // gBrushedObj.ph = 0;
            gBrushedObj.startMotor = 1;
            gBrushedObj.en_in1 = 25;
            prev_in1_duty = gBrushedObj.en_in1;// back up duty cycle setting
        }
        else
        {
            // stop
            gBrushedObj.en_in1 = 0;
            __delay_cycles(160000);            // 5ms delay for motor ramp down
            gBrushedObj.startMotor = 0;
            gBrushedObj.drvoff = 1;            // Disable driver
            if (exitflag == 1 && gInterfaceType != SPI_INTERFACE)
            {                                  // HW variant
                set_nSLEEP_lo;
                gBrushedObj.nSleep = 0;
            }
            if (gBridgeType != HALF_BRIDGE)
            {
                gBrushedObj.bridgeMode = prevBridgeMode;
            }
        }
        //P2OUT |= STATUS;                     // for debugging code
    }
}

/*
 * ADC10 interrupt Service Routine(result event) to read current value IPROPI1 & IPROPI2
 */
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{


    ipropiRaw[Adcresults] = ADC10MEM;          // Store the ADC value
    Adcresults++;
    if (Adcresults == 64)
    {
        ADC10CTL0 &= ~ENC;                     // disable ADC until moving average and peak calculation function is executed
        Adcresults = 0;
    }

    //P1OUT &= ~BIT4;  // for TP3 probing with oscilloscope
}

/*
 * TRAP: Default Service routine for all unused ISRs: Unused
 */
#pragma vector=TIMER0_A0_VECTOR, TIMER1_A0_VECTOR, \
        COMPARATORA_VECTOR, NMI_VECTOR
__interrupt void Trap_ISR(void)
{
    /*
     * reconfigure nFAULT as an output and oscillate between STATUS and nFAULT
     * LEDs to indicate the firmware has crashed
     * Note: nFAULT is active low, and STATUS is active high
     */
    P2OUT &= nSLEEP; /* disable DRV */
    P2REN &= ~nFAULT;
    P2DIR |= nFAULT;
    P2OUT &= ~nFAULT;
    P2OUT &= ~STATUS;
    while (1)
    {
        P2OUT ^= STATUS;
        P2OUT ^= nFAULT;
        _delay_cycles(10000000);
    }
}
